"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __spread = (this && this.__spread) || function () {
    for (var ar = [], i = 0; i < arguments.length; i++) ar = ar.concat(__read(arguments[i]));
    return ar;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.walksScope = exports.isFragment = exports.parseDirectives = exports.transformJSXExpressionContainer = exports.transformJSXSpreadChild = exports.transformJSXText = exports.getJSXAttributeName = exports.getTag = exports.transformJSXMemberExpression = exports.checkIsComponent = exports.isDirective = exports.createIdentifier = void 0;
var t = __importStar(require("@babel/types"));
var html_tags_1 = __importDefault(require("html-tags"));
var svg_tags_1 = __importDefault(require("svg-tags"));
/**
 * create Identifier
 * @param state
 * @param id string
 * @returns MemberExpression
 */
var createIdentifier = function (state, id) { return t.memberExpression(state.get('vue'), t.identifier(id)); };
exports.createIdentifier = createIdentifier;
/**
 * Checks if string is describing a directive
 * @param src string
 */
var isDirective = function (src) { return src.startsWith('v-')
    || (src.startsWith('v') && src.length >= 2 && src[1] >= 'A' && src[1] <= 'Z'); };
exports.isDirective = isDirective;
/**
 * Check if a Node is fragment
 * @param {*} path JSXIdentifier | JSXMemberExpression | JSXNamespacedName
 * @returns boolean
 */
var isFragment = function (path) {
    return t.isJSXMemberExpression(path)
        && path.node.property.name === 'Fragment';
};
exports.isFragment = isFragment;
/**
 * Check if a Node is a component
 *
 * @param t
 * @param path JSXOpeningElement
 * @returns boolean
 */
var checkIsComponent = function (path) {
    var namePath = path.get('name');
    if (t.isJSXMemberExpression(namePath)) {
        return !isFragment(namePath); // For withCtx
    }
    var tag = namePath.node.name;
    return !html_tags_1.default.includes(tag) && !svg_tags_1.default.includes(tag);
};
exports.checkIsComponent = checkIsComponent;
/**
 * Transform JSXMemberExpression to MemberExpression
 * @param path JSXMemberExpression
 * @returns MemberExpression
 */
var transformJSXMemberExpression = function (path) {
    var objectPath = path.node.object;
    var propertyPath = path.node.property;
    var transformedObject = t.isJSXMemberExpression(objectPath)
        ? transformJSXMemberExpression(path.get('object'))
        : t.isJSXIdentifier(objectPath)
            ? t.identifier(objectPath.name)
            : t.nullLiteral();
    var transformedProperty = t.identifier(propertyPath.name);
    return t.memberExpression(transformedObject, transformedProperty);
};
exports.transformJSXMemberExpression = transformJSXMemberExpression;
/**
 * Get tag (first attribute for h) from JSXOpeningElement
 * @param path JSXElement
 * @param state State
 * @returns Identifier | StringLiteral | MemberExpression
 */
var getTag = function (path, state) {
    var namePath = path.get('openingElement').get('name');
    if (namePath.isJSXIdentifier()) {
        var name_1 = namePath.node.name;
        if (!html_tags_1.default.includes(name_1) && !svg_tags_1.default.includes(name_1)) {
            return path.scope.hasBinding(name_1)
                ? t.identifier(name_1)
                : t.callExpression(createIdentifier(state, 'resolveComponent'), [t.stringLiteral(name_1)]);
        }
        return t.stringLiteral(name_1);
    }
    if (namePath.isJSXMemberExpression()) {
        return transformJSXMemberExpression(namePath);
    }
    throw new Error("getTag: " + namePath.type + " is not supported");
};
exports.getTag = getTag;
var getJSXAttributeName = function (path) {
    var nameNode = path.node.name;
    if (t.isJSXIdentifier(nameNode)) {
        return nameNode.name;
    }
    return nameNode.namespace.name + ":" + nameNode.name.name;
};
exports.getJSXAttributeName = getJSXAttributeName;
/**
 * Transform JSXText to StringLiteral
 * @param path JSXText
 * @returns StringLiteral | null
 */
var transformJSXText = function (path) {
    var node = path.node;
    var lines = node.value.split(/\r\n|\n|\r/);
    var lastNonEmptyLine = 0;
    for (var i = 0; i < lines.length; i++) {
        if (lines[i].match(/[^ \t]/)) {
            lastNonEmptyLine = i;
        }
    }
    var str = '';
    for (var i = 0; i < lines.length; i++) {
        var line = lines[i];
        var isFirstLine = i === 0;
        var isLastLine = i === lines.length - 1;
        var isLastNonEmptyLine = i === lastNonEmptyLine;
        // replace rendered whitespace tabs with spaces
        var trimmedLine = line.replace(/\t/g, ' ');
        // trim whitespace touching a newline
        if (!isFirstLine) {
            trimmedLine = trimmedLine.replace(/^[ ]+/, '');
        }
        // trim whitespace touching an endline
        if (!isLastLine) {
            trimmedLine = trimmedLine.replace(/[ ]+$/, '');
        }
        if (trimmedLine) {
            if (!isLastNonEmptyLine) {
                trimmedLine += ' ';
            }
            str += trimmedLine;
        }
    }
    return str !== '' ? t.stringLiteral(str) : null;
};
exports.transformJSXText = transformJSXText;
/**
 * Transform JSXExpressionContainer to Expression
 * @param path JSXExpressionContainer
 * @returns Expression
 */
var transformJSXExpressionContainer = function (path) { return path.get('expression').node; };
exports.transformJSXExpressionContainer = transformJSXExpressionContainer;
/**
 * Transform JSXSpreadChild
 * @param path JSXSpreadChild
 * @returns SpreadElement
 */
var transformJSXSpreadChild = function (path) { return t.spreadElement(path.get('expression').node); };
exports.transformJSXSpreadChild = transformJSXSpreadChild;
/**
 * Get JSX element type
 *
 * @param path Path<JSXOpeningElement>
 */
var getType = function (path) {
    var typePath = path
        .get('attributes')
        .find(function (attribute) {
        if (!t.isJSXAttribute(attribute)) {
            return false;
        }
        return t.isJSXIdentifier(attribute.get('name'))
            && attribute.get('name').get('name') === 'type'
            && t.isStringLiteral(attribute.get('value'));
    });
    return typePath ? typePath.get('value.value') : '';
};
var resolveDirective = function (path, state, tag, directiveName) {
    if (directiveName === 'show') {
        return createIdentifier(state, 'vShow');
    }
    if (directiveName === 'model') {
        var modelToUse = void 0;
        var type = getType(path.parentPath);
        switch (tag.value) {
            case 'select':
                modelToUse = createIdentifier(state, 'vModelSelect');
                break;
            case 'textarea':
                modelToUse = createIdentifier(state, 'vModelText');
                break;
            default:
                switch (type) {
                    case 'checkbox':
                        modelToUse = createIdentifier(state, 'vModelCheckbox');
                        break;
                    case 'radio':
                        modelToUse = createIdentifier(state, 'vModelRadio');
                        break;
                    default:
                        modelToUse = createIdentifier(state, 'vModelText');
                }
        }
        return modelToUse;
    }
    return t.callExpression(createIdentifier(state, 'resolveDirective'), [
        t.stringLiteral(directiveName),
    ]);
};
/**
 * Parse directives metadata
 *
 * @param  path JSXAttribute
 * @returns null | Object<{ modifiers: Set<string>, valuePath: Path<Expression>}>
 */
var parseDirectives = function (args) {
    var _a;
    var name = args.name, path = args.path, value = args.value, state = args.state, tag = args.tag, isComponent = args.isComponent;
    var modifiers = name.split('_');
    var directiveName = ((_a = modifiers.shift()) === null || _a === void 0 ? void 0 : _a.replace(/^v/, '').replace(/^-/, '').replace(/^\S/, function (s) { return s.toLowerCase(); })) || '';
    if (directiveName === 'model' && !t.isJSXExpressionContainer(path.get('value'))) {
        throw new Error('You have to use JSX Expression inside your v-model');
    }
    var modifiersSet = new Set(modifiers);
    var hasDirective = directiveName !== 'model' || (directiveName === 'model' && !isComponent);
    return {
        directiveName: directiveName,
        modifiers: modifiersSet,
        directive: hasDirective ? [
            resolveDirective(path, state, tag, directiveName),
            value,
            !!modifiersSet.size && t.unaryExpression('void', t.numericLiteral(0), true),
            !!modifiersSet.size && t.objectExpression(__spread(modifiersSet).map(function (modifier) { return t.objectProperty(t.identifier(modifier), t.booleanLiteral(true)); })),
        ].filter(Boolean) : undefined,
    };
};
exports.parseDirectives = parseDirectives;
var walksScope = function (path, name) {
    if (path.scope.hasBinding(name)) {
        path.parentPath.setData('optimize', false);
        walksScope(path.parentPath, name);
    }
};
exports.walksScope = walksScope;
